<!--startmeta
custom_edit_url: "https://github.com/netdata/go.d.plugin/edit/master/modules/apache/integrations/apache.md"
meta_yaml: "https://github.com/netdata/go.d.plugin/edit/master/modules/apache/metadata.yaml"
sidebar_label: "Apache"
learn_status: "Published"
learn_rel_path: "Data Collection/Web Servers and Web Proxies"
most_popular: True
message: "DO NOT EDIT THIS FILE DIRECTLY, IT IS GENERATED BY THE COLLECTOR'S metadata.yaml FILE"
endmeta-->

# Apache


<img src="https://netdata.cloud/img/apache.svg" width="150"/>


Plugin: go.d.plugin
Module: apache

<img src="https://img.shields.io/badge/maintained%20by-Netdata-%2300ab44" />

## Overview

This collector monitors the activity and performance of Apache servers, and collects metrics such as the number of connections, workers, requests and more.


It sends HTTP requests to the Apache location [server-status](https://httpd.apache.org/docs/2.4/mod/mod_status.html), 
which is a built-in location that provides metrics about the Apache server.


This collector is supported on all platforms.

This collector supports collecting metrics from multiple instances of this integration, including remote instances.


### Default Behavior

#### Auto-Detection

By default, it detects Apache instances running on localhost that are listening on port 80.
On startup, it tries to collect metrics from:

- http://localhost/server-status?auto
- http://127.0.0.1/server-status?auto


#### Limits

The default configuration for this integration does not impose any limits on data collection.

#### Performance Impact

The default configuration for this integration is not expected to impose a significant performance impact on the system.


## Metrics

Metrics grouped by *scope*.

The scope defines the instance that the metric belongs to. An instance is uniquely identified by a set of labels.

All metrics available only if [ExtendedStatus](https://httpd.apache.org/docs/2.4/mod/core.html#extendedstatus) is on.


### Per Apache instance

These metrics refer to the entire monitored application.

This scope has no labels.

Metrics:

| Metric | Dimensions | Unit | Basic | Extended |
|:------|:----------|:----|:---:|:---:|
| apache.connections | connections | connections | • | • |
| apache.conns_async | keepalive, closing, writing | connections | • | • |
| apache.workers | idle, busy | workers | • | • |
| apache.scoreboard | waiting, starting, reading, sending, keepalive, dns_lookup, closing, logging, finishing, idle_cleanup, open | connections | • | • |
| apache.requests | requests | requests/s |   | • |
| apache.net | sent | kilobit/s |   | • |
| apache.reqpersec | requests | requests/s |   | • |
| apache.bytespersec | served | KiB/s |   | • |
| apache.bytesperreq | size | KiB |   | • |
| apache.uptime | uptime | seconds |   | • |



## Alerts

There are no alerts configured by default for this integration.


## Setup

### Prerequisites

#### Enable Apache status support

- Enable and configure [status_module](https://httpd.apache.org/docs/2.4/mod/mod_status.html).
- Ensure that you have [ExtendedStatus](https://httpd.apache.org/docs/2.4/mod/mod_status.html#troubleshoot) set on (enabled by default since Apache v2.3.6).



### Configuration

#### File

The configuration file name for this integration is `go.d/apache.conf`.


You can edit the configuration file using the `edit-config` script from the
Netdata [config directory](https://github.com/netdata/netdata/blob/master/docs/configure/nodes.md#the-netdata-config-directory).

```bash
cd /etc/netdata 2>/dev/null || cd /opt/netdata/etc/netdata
sudo ./edit-config go.d/apache.conf
```
#### Options

The following options can be defined globally: update_every, autodetection_retry.


<details><summary>Config options</summary>

| Name | Description | Default | Required |
|:----|:-----------|:-------|:--------:|
| update_every | Data collection frequency. | 1 | no |
| autodetection_retry | Recheck interval in seconds. Zero means no recheck will be scheduled. | 0 | no |
| url | Server URL. | http://127.0.0.1/server-status?auto | yes |
| timeout | HTTP request timeout. | 1 | no |
| username | Username for basic HTTP authentication. |  | no |
| password | Password for basic HTTP authentication. |  | no |
| proxy_url | Proxy URL. |  | no |
| proxy_username | Username for proxy basic HTTP authentication. |  | no |
| proxy_password | Password for proxy basic HTTP authentication. |  | no |
| method | HTTP request method. | GET | no |
| body | HTTP request body. |  | no |
| headers | HTTP request headers. |  | no |
| not_follow_redirects | Redirect handling policy. Controls whether the client follows redirects. | no | no |
| tls_skip_verify | Server certificate chain and hostname validation policy. Controls whether the client performs this check. | no | no |
| tls_ca | Certification authority that the client uses when verifying the server's certificates. |  | no |
| tls_cert | Client TLS certificate. |  | no |
| tls_key | Client TLS key. |  | no |

</details>

#### Examples

##### Basic

A basic example configuration.

```yaml
jobs:
  - name: local
    url: http://127.0.0.1/server-status?auto

```
##### HTTP authentication

Basic HTTP authentication.

<details><summary>Config</summary>

```yaml
jobs:
  - name: local
    url: http://127.0.0.1/server-status?auto
    username: username
    password: password

```
</details>

##### HTTPS with self-signed certificate

Apache with enabled HTTPS and self-signed certificate.

<details><summary>Config</summary>

```yaml
jobs:
  - name: local
    url: https://127.0.0.1/server-status?auto
    tls_skip_verify: yes

```
</details>

##### Multi-instance

> **Note**: When you define multiple jobs, their names must be unique.

Collecting metrics from local and remote instances.


<details><summary>Config</summary>

```yaml
jobs:
  - name: local
    url: http://127.0.0.1/server-status?auto

  - name: remote
    url: http://192.0.2.1/server-status?auto

```
</details>



## Troubleshooting

### Debug Mode

To troubleshoot issues with the `apache` collector, run the `go.d.plugin` with the debug option enabled. The output
should give you clues as to why the collector isn't working.

- Navigate to the `plugins.d` directory, usually at `/usr/libexec/netdata/plugins.d/`. If that's not the case on
  your system, open `netdata.conf` and look for the `plugins` setting under `[directories]`.

  ```bash
  cd /usr/libexec/netdata/plugins.d/
  ```

- Switch to the `netdata` user.

  ```bash
  sudo -u netdata -s
  ```

- Run the `go.d.plugin` to debug the collector:

  ```bash
  ./go.d.plugin -d -m apache
  ```


