<!--startmeta
custom_edit_url: "https://github.com/netdata/go.d.plugin/edit/master/modules/dnsmasq_dhcp/README.md"
meta_yaml: "https://github.com/netdata/go.d.plugin/edit/master/modules/dnsmasq_dhcp/metadata.yaml"
sidebar_label: "Dnsmasq DHCP"
learn_status: "Published"
learn_rel_path: "Data Collection/DNS and DHCP Servers"
most_popular: False
message: "DO NOT EDIT THIS FILE DIRECTLY, IT IS GENERATED BY THE COLLECTOR'S metadata.yaml FILE"
endmeta-->

# Dnsmasq DHCP


<img src="https://netdata.cloud/img/dnsmasq.svg" width="150"/>


Plugin: go.d.plugin
Module: dnsmasq_dhcp

<img src="https://img.shields.io/badge/maintained%20by-Netdata-%2300ab44" />

## Overview

This collector monitors  Dnsmasq DHCP leases databases, depending on your configuration.

By default, it uses:

- `/var/lib/misc/dnsmasq.leases` to read leases.
- `/etc/dnsmasq.conf` to detect dhcp-ranges.
- `/etc/dnsmasq.d` to find additional configurations.




This collector is supported on all platforms.

This collector only supports collecting metrics from a single instance of this integration.


### Default Behavior

#### Auto-Detection

All configured dhcp-ranges are detected automatically


#### Limits

The default configuration for this integration does not impose any limits on data collection.

#### Performance Impact

The default configuration for this integration is not expected to impose a significant performance impact on the system.


## Metrics

Metrics grouped by *scope*.

The scope defines the instance that the metric belongs to. An instance is uniquely identified by a set of labels.



### Per Dnsmasq DHCP instance

These metrics refer to the entire monitored application.

This scope has no labels.

Metrics:

| Metric | Dimensions | Unit |
|:------|:----------|:----|
| dnsmasq_dhcp.dhcp_ranges | ipv4, ipv6 | ranges |
| dnsmasq_dhcp.dhcp_hosts | ipv4, ipv6 | hosts |

### Per dhcp range

These metrics refer to the DHCP range.

Labels:

| Label      | Description     |
|:-----------|:----------------|
| dhcp_range | DHCP range in `START_IP:END_IP` format |

Metrics:

| Metric | Dimensions | Unit |
|:------|:----------|:----|
| dnsmasq_dhcp.dhcp_range_utilization | used | percentage |
| dnsmasq_dhcp.dhcp_range_allocated_leases | allocated | leases |



## Alerts


The following alerts are available:

| Alert name  | On metric | Description |
|:------------|:----------|:------------|
| [ dnsmasq_dhcp_dhcp_range_utilization ](https://github.com/netdata/netdata/blob/master/health/health.d/dnsmasq_dhcp.conf) | dnsmasq_dhcp.dhcp_range_utilization | DHCP range utilization |


## Setup

### Prerequisites

No action required.

### Configuration

#### File

The configuration file name for this integration is `go.d/dnsmasq_dhcp.conf`.


You can edit the configuration file using the `edit-config` script from the
Netdata [config directory](https://github.com/netdata/netdata/blob/master/docs/configure/nodes.md#the-netdata-config-directory).

```bash
cd /etc/netdata 2>/dev/null || cd /opt/netdata/etc/netdata
sudo ./edit-config go.d/dnsmasq_dhcp.conf
```
#### Options

The following options can be defined globally: update_every, autodetection_retry.


<details><summary>Config options</summary>

| Name | Description | Default | Required |
|:----|:-----------|:-------|:--------:|
| update_every | Data collection frequency. | 1 | no |
| autodetection_retry | Recheck interval in seconds. Zero means no recheck will be scheduled. | 0 | no |
| leases_path | Path to dnsmasq DHCP leases file. | /var/lib/misc/dnsmasq.leases | no |
| conf_path | Path to dnsmasq configuration file. | /etc/dnsmasq.conf | no |
| conf_dir | Path to dnsmasq configuration directory. | /etc/dnsmasq.d,.dpkg-dist,.dpkg-old,.dpkg-new | no |

</details>

#### Examples

##### Basic

An example configuration.

<details><summary>Config</summary>

```yaml
jobs:
  - name: dnsmasq_dhcp
    leases_path: /var/lib/misc/dnsmasq.leases
    conf_path: /etc/dnsmasq.conf
    conf_dir: /etc/dnsmasq.d

```
</details>

##### Pi-hole

Dnsmasq DHCP on Pi-hole.

<details><summary>Config</summary>

```yaml
jobs:
  - name: dnsmasq_dhcp
    leases_path: /etc/pihole/dhcp.leases
    conf_path: /etc/dnsmasq.conf
    conf_dir: /etc/dnsmasq.d

```
</details>



## Troubleshooting

### Debug Mode

To troubleshoot issues with the `dnsmasq_dhcp` collector, run the `go.d.plugin` with the debug option enabled. The output
should give you clues as to why the collector isn't working.

- Navigate to the `plugins.d` directory, usually at `/usr/libexec/netdata/plugins.d/`. If that's not the case on
  your system, open `netdata.conf` and look for the `plugins` setting under `[directories]`.

  ```bash
  cd /usr/libexec/netdata/plugins.d/
  ```

- Switch to the `netdata` user.

  ```bash
  sudo -u netdata -s
  ```

- Run the `go.d.plugin` to debug the collector:

  ```bash
  ./go.d.plugin -d -m dnsmasq_dhcp
  ```


