<!--startmeta
custom_edit_url: "https://github.com/netdata/go.d.plugin/edit/master/modules/filecheck/README.md"
meta_yaml: "https://github.com/netdata/go.d.plugin/edit/master/modules/filecheck/metadata.yaml"
sidebar_label: "Files and directories"
learn_status: "Published"
learn_rel_path: "Data Collection/Linux Systems"
most_popular: False
message: "DO NOT EDIT THIS FILE DIRECTLY, IT IS GENERATED BY THE COLLECTOR'S metadata.yaml FILE"
endmeta-->

# Files and directories


<img src="https://netdata.cloud/img/filesystem.svg" width="150"/>


Plugin: go.d.plugin
Module: filecheck

<img src="https://img.shields.io/badge/maintained%20by-Netdata-%2300ab44" />

## Overview

This collector monitors files and directories.




This collector is supported on all platforms.

This collector supports collecting metrics from multiple instances of this integration, including remote instances.

`netdata` user needs the following permissions on all the directories in pathname that lead to the file/dir:

- files monitoring: `execute`.
- directories monitoring: `read` and `execute`.

If you need to modify the permissions we
suggest [to use file access control lists](https://linux.die.net/man/1/setfacl):

```cmd
setfacl -m u:netconfig:rx file ...
``` 

> **Warning**: For security reasons, this should not be applied recursively, but only to the exact set of directories
> that lead to the file/dir you want to monitor.


### Default Behavior

#### Auto-Detection

This integration doesn't support auto-detection.

#### Limits

The default configuration for this integration does not impose any limits on data collection.

#### Performance Impact

The default configuration for this integration is not expected to impose a significant performance impact on the system.


## Metrics

Metrics grouped by *scope*.

The scope defines the instance that the metric belongs to. An instance is uniquely identified by a set of labels.



### Per Files and directories instance

TBD

This scope has no labels.

Metrics:

| Metric | Dimensions | Unit |
|:------|:----------|:----|
| filecheck.file_existence | a dimension per file | boolean |
| filecheck.file_mtime_ago | a dimension per file | seconds |
| filecheck.file_size | a dimension per file | bytes |
| filecheck.dir_existence | a dimension per directory | boolean |
| filecheck.dir_mtime_ago | a dimension per directory | seconds |
| filecheck.dir_num_of_files | a dimension per directory | files |
| filecheck.dir_size | a dimension per directory | bytes |



## Alerts

There are no alerts configured by default for this integration.


## Setup

### Prerequisites

No action required.

### Configuration

#### File

The configuration file name for this integration is `go.d/filecheck.conf`.


You can edit the configuration file using the `edit-config` script from the
Netdata [config directory](https://github.com/netdata/netdata/blob/master/docs/configure/nodes.md#the-netdata-config-directory).

```bash
cd /etc/netdata 2>/dev/null || cd /opt/netdata/etc/netdata
sudo ./edit-config go.d/filecheck.conf
```
#### Options

The following options can be defined globally: update_every, autodetection_retry.


<details><summary>Config options</summary>

| Name | Description | Default | Required |
|:----|:-----------|:-------|:--------:|
| update_every | Data collection frequency. | 10 | no |
| autodetection_retry | Recheck interval in seconds. Zero means no recheck will be scheduled. | 0 | no |
| files | List of files to monitor. |  | yes |
| dirs | List of directories to monitor. |  | yes |
| discovery_every | Files and directories discovery interval. | 60 | no |

##### files

Files matching the selector will be monitored.

- Logic: (pattern1 OR pattern2) AND !(pattern3 or pattern4)
- Pattern syntax: [shell file name pattern](https://golang.org/pkg/path/filepath/#Match)
- Syntax:

```yaml
files:
  includes:
    - pattern1
    - pattern2
  excludes:
    - pattern3
    - pattern4
```


##### dirs

Directories matching the selector will be monitored.

- Logic: (pattern1 OR pattern2) AND !(pattern3 or pattern4)
- Pattern syntax: [shell file name pattern](https://golang.org/pkg/path/filepath/#Match)
- Syntax:

```yaml
dirs:
  includes:
    - pattern1
    - pattern2
  excludes:
    - pattern3
    - pattern4
```


</details>

#### Examples

##### Files

Files monitoring example configuration.

<details><summary>Config</summary>

```yaml
jobs:
  - name: files_example
    files:
      include:
        - '/path/to/file1'
        - '/path/to/file2'
        - '/path/to/*.log'

```
</details>

##### Directories

Directories monitoring example configuration.

<details><summary>Config</summary>

```yaml
jobs:
  - name: files_example
    dirs:
      collect_dir_size: no
      include:
        - '/path/to/dir1'
        - '/path/to/dir2'
        - '/path/to/dir3*'

```
</details>



## Troubleshooting

### Debug Mode

To troubleshoot issues with the `filecheck` collector, run the `go.d.plugin` with the debug option enabled. The output
should give you clues as to why the collector isn't working.

- Navigate to the `plugins.d` directory, usually at `/usr/libexec/netdata/plugins.d/`. If that's not the case on
  your system, open `netdata.conf` and look for the `plugins` setting under `[directories]`.

  ```bash
  cd /usr/libexec/netdata/plugins.d/
  ```

- Switch to the `netdata` user.

  ```bash
  sudo -u netdata -s
  ```

- Run the `go.d.plugin` to debug the collector:

  ```bash
  ./go.d.plugin -d -m filecheck
  ```


