<!--startmeta
custom_edit_url: "https://github.com/netdata/go.d.plugin/edit/master/modules/phpfpm/README.md"
meta_yaml: "https://github.com/netdata/go.d.plugin/edit/master/modules/phpfpm/metadata.yaml"
sidebar_label: "PHP-FPM"
learn_status: "Published"
learn_rel_path: "Data Collection/Web Servers and Web Proxies"
most_popular: False
message: "DO NOT EDIT THIS FILE DIRECTLY, IT IS GENERATED BY THE COLLECTOR'S metadata.yaml FILE"
endmeta-->

# PHP-FPM


<img src="https://netdata.cloud/img/php.svg" width="150"/>


Plugin: go.d.plugin
Module: phpfpm

<img src="https://img.shields.io/badge/maintained%20by-Netdata-%2300ab44" />

## Overview

This collector monitors PHP-FPM instances.




This collector is supported on all platforms.

This collector supports collecting metrics from multiple instances of this integration, including remote instances.


### Default Behavior

#### Auto-Detection

This integration doesn't support auto-detection.

#### Limits

The default configuration for this integration does not impose any limits on data collection.

#### Performance Impact

The default configuration for this integration is not expected to impose a significant performance impact on the system.


## Metrics

Metrics grouped by *scope*.

The scope defines the instance that the metric belongs to. An instance is uniquely identified by a set of labels.



### Per PHP-FPM instance

These metrics refer to the entire monitored application.

This scope has no labels.

Metrics:

| Metric | Dimensions | Unit |
|:------|:----------|:----|
| phpfpm.connections | active, max_active, idle | connections |
| phpfpm.requests | requests | requests/s |
| phpfpm.performance | max_children_reached, slow_requests | status |
| phpfpm.request_duration | min, max, avg | milliseconds |
| phpfpm.request_cpu | min, max, avg | percentage |
| phpfpm.request_mem | min, max, avg | KB |



## Alerts

There are no alerts configured by default for this integration.


## Setup

### Prerequisites

#### Enable status page

Uncomment the `pm.status_path = /status` variable in the `php-fpm` config file.



### Configuration

#### File

The configuration file name for this integration is `go.d/phpfpm.conf`.


You can edit the configuration file using the `edit-config` script from the
Netdata [config directory](https://github.com/netdata/netdata/blob/master/docs/configure/nodes.md#the-netdata-config-directory).

```bash
cd /etc/netdata 2>/dev/null || cd /opt/netdata/etc/netdata
sudo ./edit-config go.d/phpfpm.conf
```
#### Options

The following options can be defined globally: update_every, autodetection_retry.


<details><summary>Config options</summary>

| Name | Description | Default | Required |
|:----|:-----------|:-------|:--------:|
| update_every | Data collection frequency. | 1 | no |
| autodetection_retry | Recheck interval in seconds. Zero means no recheck will be scheduled. | 0 | no |
| url | Server URL. | http://127.0.0.1/status?full&json | yes |
| socket | Server Unix socket. |  | no |
| address | Server address in IP:PORT format. |  | no |
| fcgi_path | Status path. | /status | no |
| timeout | HTTP request timeout. | 1 | no |
| username | Username for basic HTTP authentication. |  | no |
| password | Password for basic HTTP authentication. |  | no |
| proxy_url | Proxy URL. |  | no |
| proxy_username | Username for proxy basic HTTP authentication. |  | no |
| proxy_password | Password for proxy basic HTTP authentication. |  | no |
| method | HTTP request method. | GET | no |
| body | HTTP request body. |  | no |
| headers | HTTP request headers. |  | no |
| not_follow_redirects | Redirect handling policy. Controls whether the client follows redirects. | no | no |
| tls_skip_verify | Server certificate chain and hostname validation policy. Controls whether the client performs this check. | no | no |
| tls_ca | Certification authority that the client uses when verifying the server's certificates. |  | no |
| tls_cert | Client TLS certificate. |  | no |
| tls_key | Client TLS key. |  | no |

</details>

#### Examples

##### HTTP

Collecting data from a local instance over HTTP.

<details><summary>Config</summary>

```yaml
jobs:
  - name: local
    url: http://localhost/status?full&json

```
</details>

##### Unix socket

Collecting data from a local instance over Unix socket.

<details><summary>Config</summary>

```yaml
jobs:
  - name: local
    socket: '/tmp/php-fpm.sock'

```
</details>

##### TCP socket

Collecting data from a local instance over TCP socket.

<details><summary>Config</summary>

```yaml
jobs:
  - name: local
    address: 127.0.0.1:9000

```
</details>

##### Multi-instance

> **Note**: When you define multiple jobs, their names must be unique.

Collecting metrics from local and remote instances.


<details><summary>Config</summary>

```yaml
jobs:
    - name: local
      url: http://localhost/status?full&json

    - name: remote
      url: http://203.0.113.10/status?full&json

```
</details>



## Troubleshooting

### Debug Mode

To troubleshoot issues with the `phpfpm` collector, run the `go.d.plugin` with the debug option enabled. The output
should give you clues as to why the collector isn't working.

- Navigate to the `plugins.d` directory, usually at `/usr/libexec/netdata/plugins.d/`. If that's not the case on
  your system, open `netdata.conf` and look for the `plugins` setting under `[directories]`.

  ```bash
  cd /usr/libexec/netdata/plugins.d/
  ```

- Switch to the `netdata` user.

  ```bash
  sudo -u netdata -s
  ```

- Run the `go.d.plugin` to debug the collector:

  ```bash
  ./go.d.plugin -d -m phpfpm
  ```


